from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import (
    MODEL,
    NORMAL,
    OFF,
    ON,
    Boolean,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .DesignResponse import DesignResponse
from .GeometricRestriction import GeometricRestriction
from .ObjectiveFunction import ObjectiveFunction
from .OptimizationConstraint import OptimizationConstraint
from .OptimizationTask import OptimizationTask
from .StopCondition import StopCondition


@abaqus_class_doc
class SizingTask(OptimizationTask):
    """The SizingTask object defines a Sizing task. The SizingTask object is derived from the OptimizationTask
    object.

    .. note::
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name]
    """

    #: A repository of DesignResponse objects.
    designResponses: dict[str, DesignResponse] = {}

    #: A repository of ObjectiveFunction objects.
    objectiveFunctions: dict[str, ObjectiveFunction] = {}

    #: A repository of OptimizationConstraint objects.
    optimizationConstraints: dict[str, OptimizationConstraint] = {}

    #: A repository of GeometricRestriction objects.
    geometricRestrictions: dict[str, GeometricRestriction] = {}

    #: A repository of StopCondition objects.
    stopConditions: dict[str, StopCondition] = {}

    #: A String specifying the optimization task repository key.
    name: str

    #: A Float specifying the stop criteria based on the change in element thickness. The
    #: default value is 0.5 x 10⁻².
    elementThicknessDeltaStopCriteria: float = 0

    #: A Boolean specifying whether to exclude elements with boundary conditions from the
    #: optimization. The default value is OFF.
    freezeBoundaryConditionRegions: Boolean = OFF

    #: A Boolean specifying whether to exclude elements with loads and elements with loaded
    #: nodes from the optimization. The default value is ON.
    freezeLoadRegions: Boolean = ON

    #: The SymbolicConstatnt MODEL or a Region object specifying the region to use for mode
    #: tracking. The default value is MODEL.
    modeTrackingRegion: str = MODEL

    #: An Int specifying the number of stop criteria. The default value is 2.
    numFulfilledStopCriteria: int = 2

    #: An Int specifying the number of modes included in mode tracking. The default value is 5.
    numTrackedModes: int = 5

    #: A Float specifying the stop criteria based on the change in objective function. The
    #: default value is 0.001.
    objectiveFunctionDeltaStopCriteria: float = 0

    #: An Int specifying the first design cycle used to evaluate convergence criteria. The
    #: default value is 4.
    stopCriteriaDesignCycle: int = 4

    #: A Float specifying the maximum change in thickness per design cycle. The default value
    #: is 0.25.
    thicknessMoveLimit: float = 0

    #: A SymbolicConstant specifying the strategy for how the thickness is updated in the
    #: method of moving asymptotes. Possible values are NORMAL, CONSERVATIVE, and AGGRESSIVE.
    #: The default value is NORMAL.
    thicknessUpdateStrategy: SymbolicConstant = NORMAL

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        elementThicknessDeltaStopCriteria: float = 0,
        freezeBoundaryConditionRegions: Boolean = OFF,
        freezeLoadRegions: Boolean = ON,
        modeTrackingRegion: str = MODEL,
        numFulfilledStopCriteria: int = 2,
        numTrackedModes: int = 5,
        objectiveFunctionDeltaStopCriteria: float = 0,
        stopCriteriaDesignCycle: int = 4,
        thicknessMoveLimit: float = 0,
        thicknessUpdateStrategy: Literal[C.AGGRESSIVE, C.NORMAL, C.CONSERVATIVE] = NORMAL,
    ):
        """This method creates a SizingTask object.

        .. note::
            This function can be accessed by::

                mdb.models[name].SizingTask

        Parameters
        ----------
        name
            A String specifying the optimization task repository key.
        elementThicknessDeltaStopCriteria
            A Float specifying the stop criteria based on the change in element thickness. The
            default value is 0.5 x 10⁻².
        freezeBoundaryConditionRegions
            A Boolean specifying whether to exclude elements with boundary conditions from the
            optimization. The default value is OFF.
        freezeLoadRegions
            A Boolean specifying whether to exclude elements with loads and elements with loaded
            nodes from the optimization. The default value is ON.
        modeTrackingRegion
            The SymbolicConstatnt MODEL or a Region object specifying the region to use for mode
            tracking. The default value is MODEL.
        numFulfilledStopCriteria
            An Int specifying the number of stop criteria. The default value is 2.
        numTrackedModes
            An Int specifying the number of modes included in mode tracking. The default value is 5.
        objectiveFunctionDeltaStopCriteria
            A Float specifying the stop criteria based on the change in objective function. The
            default value is 0.001.
        stopCriteriaDesignCycle
            An Int specifying the first design cycle used to evaluate convergence criteria. The
            default value is 4.
        thicknessMoveLimit
            A Float specifying the maximum change in thickness per design cycle. The default value
            is 0.25.
        thicknessUpdateStrategy
            A SymbolicConstant specifying the strategy for how the thickness is updated in the
            method of moving asymptotes. Possible values are NORMAL, CONSERVATIVE, and AGGRESSIVE.
            The default value is NORMAL.

        Returns
        -------
        SizingTask
            A SizingTask object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        elementThicknessDeltaStopCriteria: float = 0,
        freezeBoundaryConditionRegions: Boolean = OFF,
        freezeLoadRegions: Boolean = ON,
        modeTrackingRegion: str = MODEL,
        numFulfilledStopCriteria: int = 2,
        numTrackedModes: int = 5,
        objectiveFunctionDeltaStopCriteria: float = 0,
        stopCriteriaDesignCycle: int = 4,
        thicknessMoveLimit: float = 0,
        thicknessUpdateStrategy: Literal[C.AGGRESSIVE, C.NORMAL, C.CONSERVATIVE] = NORMAL,
    ):
        """This method modifies the SizingTask object.

        Parameters
        ----------
        elementThicknessDeltaStopCriteria
            A Float specifying the stop criteria based on the change in element thickness. The
            default value is 0.5 x 10⁻².
        freezeBoundaryConditionRegions
            A Boolean specifying whether to exclude elements with boundary conditions from the
            optimization. The default value is OFF.
        freezeLoadRegions
            A Boolean specifying whether to exclude elements with loads and elements with loaded
            nodes from the optimization. The default value is ON.
        modeTrackingRegion
            The SymbolicConstatnt MODEL or a Region object specifying the region to use for mode
            tracking. The default value is MODEL.
        numFulfilledStopCriteria
            An Int specifying the number of stop criteria. The default value is 2.
        numTrackedModes
            An Int specifying the number of modes included in mode tracking. The default value is 5.
        objectiveFunctionDeltaStopCriteria
            A Float specifying the stop criteria based on the change in objective function. The
            default value is 0.001.
        stopCriteriaDesignCycle
            An Int specifying the first design cycle used to evaluate convergence criteria. The
            default value is 4.
        thicknessMoveLimit
            A Float specifying the maximum change in thickness per design cycle. The default value
            is 0.25.
        thicknessUpdateStrategy
            A SymbolicConstant specifying the strategy for how the thickness is updated in the
            method of moving asymptotes. Possible values are NORMAL, CONSERVATIVE, and AGGRESSIVE.
            The default value is NORMAL.
        """
        ...



from __future__ import annotations

import os
from importlib import import_module, reload

import pytest
from typing_extensions import Literal

config_module = import_module("abqpy.config")


@pytest.mark.parametrize(
    argnames="type, envs, expects",
    argvalues=[
        ("cae", dict(ABAQUS_COMMAND_OPTIONS="{'noGUI': True}"), dict(gui=False)),
        ("cae", dict(ABAQUS_COMMAND_OPTIONS="{'noGUI': True}", ABAQUS_CAE_GUI="true"), dict(gui=True)),
        ("cae", dict(ABAQUS_CAE_GUI="no"), dict(gui=False)),
        ("cae", dict(ABAQUS_CAE_DATABASE="test.odb"), dict(database="test.odb")),
        ("cae", dict(ABAQUS_CAE_REPLAY="test.rpy"), dict(replay="test.rpy")),
        ("cae", dict(ABAQUS_CAE_RECOVER="test.rpy"), dict(recover="test.rpy")),
        ("cae", dict(ABAQUS_CAE_ENVSTARTUP="off"), dict(envstartup=False)),
        ("cae", dict(ABAQUS_CAE_SAVEDOPTIONS="off"), dict(savedOptions=False)),
        ("cae", dict(ABAQUS_CAE_SAVEDGUIPREFS="off"), dict(savedGuiPrefs=False)),
        ("cae", dict(ABAQUS_CAE_STARTUPDIALOG="off"), dict(startupDialog=False)),
        ("cae", dict(ABAQUS_CAE_CUSTOM="test.py"), dict(custom="test.py")),
        ("cae", dict(ABAQUS_CAE_GUITESTER="test.py"), dict(guiTester="test.py")),
        ("python", dict(ABAQUS_PYTHON_SIM="test.py"), dict(sim="test.py")),
        ("python", dict(ABAQUS_PYTHON_LOG="test.log"), dict(log="test.log")),
    ],
    ids=["GUI", "GUI override", "GUI off", "database", "replay", "recover", "envstartup", "savedOptions",
         "savedGuiPrefs", "startupDialog", "custom", "guiTester", "sim", "log"],  # fmt: skip
)
def test_config(type: Literal["cae", "python"], envs: dict[str, str], expects: dict[str, str | bool]):
    for key, env in envs.items():
        os.environ[key] = env

    config = reload(config_module)
    for key, expect in expects.items():
        actual = getattr(config.config.cae if type == "cae" else config.config.python, key)
        assert actual == expect, f"Expected {key} to be {expect}, got {actual}"

    for key, env in envs.items():
        del os.environ[key]



from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class ContactControl:
    """The ContactControl object is the abstract base type for other ContactControl objects. The ContactControl
    object has no explicit constructor, members, or methods.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].contactControls[name]
    """

    ...



from __future__ import annotations


class FeatureSketch:
    ...


def Sketch(self, sheetSize: float = 200, gridSize: float = 5) -> FeatureSketch:
    return FeatureSketch()


def SketchTransform():
    ...


__all__ = [
    "Sketch",
    "SketchTransform",
]



from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import (
    LINEAR,
    OFF,
    UNCOUPLED,
    Boolean,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .ConnectorBehaviorOption import ConnectorBehaviorOption
from .ConnectorOptions import ConnectorOptions


@abaqus_class_doc
class ConnectorDamping(ConnectorBehaviorOption):
    """The ConnectorDamping object defines damping behavior for one or more components of a connector's relative
    motion. The ConnectorDamping object is derived from the ConnectorBehaviorOption object.

    .. note::
        This object can be accessed by::

            import section
            mdb.models[name].sections[name].behaviorOptions[i]
            import odbSection
            session.odbs[name].sections[name].behaviorOptions[i]

        The table data for this object are:

        - If **behavior** = LINEAR and **coupling** = UNCOUPLED, then each sequence of the table data specifies the following:

            - Damping coefficient (force or moment per relative velocity).
            - Frequency (cycles/time), if the data depend on frequency.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        - If **behavior** = NONLINEAR and **coupling** = UNCOUPLED, then each sequence of the table data specifies the following:

            - Force or moment.
            - Relative displacement or rotation.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        - If **behavior** = LINEAR and **coupling** = COUPLED, the table data specify the symmetric portion
            of the damping matrix for the specified components followed by any temperature data and then
            any field data. For example, if components 2, 3, and 5 are specified, the table portion of the command is as follows: table=( (C22C22, C23C23, C25C25, C33C33, C35C35, C55C55,), )
            The following items should then be specified as comma-separated data:

            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        - If **behavior** = NONLINEAR and **coupling** = COUPLED_POSITION or COUPLED_MOTION, each sequence
            of the table data specifies the following:

            - Force or moment for the directions in the **components** list.
            - Relative velocity for the directions in the **components** list.
            - Relative position or angle for the first **independentComponents** direction if
                **coupling** = COUPLED_POSITION.
            - Relative displacement or rotation for the first **independentComponents** direction if
                **coupling** = COUPLED_MOTION.
            - Relative position or angle for the second **independentComponents** direction if
                **coupling** = COUPLED_POSITION.
            - Relative displacement or rotation for the second **independentComponents** direction if
                **coupling** = COUPLED_MOTION.
            - Etc. up to the Nth **independentComponents** direction.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

        The corresponding analysis keywords are:

        - CONNECTOR DAMPING
    """

    #: A ConnectorOptions object specifying the ConnectorOptions used to define tabular options
    #: for this ConnectorBehaviorOption.
    options: ConnectorOptions = ConnectorOptions()

    #: A SymbolicConstant specifying if the damping behavior is linear or nonlinear. Possible
    #: values are LINEAR and NONLINEAR. The default value is LINEAR.
    behavior: SymbolicConstant = LINEAR

    #: A SymbolicConstant specifying whether the damping behavior is coupled between the
    #: connector's components of relative motion. If **behavior** = LINEAR, then possible values
    #: are UNCOUPLED and COUPLED. If **behavior** = NONLINEAR, then possible values are UNCOUPLED,
    #: COUPLED_POSITION, and COUPLED_MOTION. Possible values are UNCOUPLED, COUPLED,
    #: COUPLED_POSITION, and COUPLED_MOTION. The default value is UNCOUPLED.
    coupling: SymbolicConstant = UNCOUPLED

    #: An Int specifying the number of field variable dependencies. The default value is 0.
    dependencies: int = 0

    #: A Boolean specifying whether the behavior data depend on temperature. The default value
    #: is OFF.
    temperatureDependency: Boolean = OFF

    #: A Boolean specifying whether the behavior data depend on frequency. This value is
    #: applicable only if **behavior** =  LINEAR and **coupling** = UNCOUPLED. The default value is
    #: OFF.
    frequencyDependency: Boolean = OFF

    #: A sequence of sequences of Floats specifying damping properties. Items in the table data
    #: are described below. The default value is an empty sequence.
    table: tuple = ()

    #: A sequence of Ints specifying the list of independent components that are included in
    #: the definition of the connector damping data. This argument is applicable only if
    #: **behavior** = NONLINEAR and **coupling** = COUPLED_POSITION or COUPLED_MOTION. When this
    #: argument is applicable, at least one value must be specified. Only available components
    #: can be specified. The default value is an empty sequence.
    independentComponents: tuple = ()

    #: A sequence of Ints specifying the components of relative motion for which the behavior
    #: is defined. Possible values are 1 ≤ **components** ≤ 6. Only available components can be
    #: specified. The default value is an empty sequence.
    components: tuple = ()

    @abaqus_method_doc
    def __init__(
        self,
        behavior: Literal[C.NONLINEAR, C.LINEAR] = LINEAR,
        coupling: Literal[
            C.UNCOUPLED, C.NONLINEAR, C.COUPLED_POSITION, C.COUPLED_MOTION, C.COUPLED, C.LINEAR
        ] = UNCOUPLED,
        dependencies: int = 0,
        temperatureDependency: Boolean = OFF,
        frequencyDependency: Boolean = OFF,
        table: tuple = (),
        independentComponents: tuple = (),
        components: tuple = (),
    ):
        """This method creates a connector damping behavior option for a ConnectorSection object.

        .. note::
            This function can be accessed by::

                import connectorBehavior
                connectorBehavior.ConnectorDamping
                import odbConnectorBehavior
                odbConnectorBehavior.ConnectorDamping

        Parameters
        ----------
        behavior
            A SymbolicConstant specifying if the damping behavior is linear or nonlinear. Possible
            values are LINEAR and NONLINEAR. The default value is LINEAR.
        coupling
            A SymbolicConstant specifying whether the damping behavior is coupled between the
            connector's components of relative motion. If **behavior** = LINEAR, then possible values
            are UNCOUPLED and COUPLED. If **behavior** = NONLINEAR, then possible values are UNCOUPLED,
            COUPLED_POSITION, and COUPLED_MOTION. Possible values are UNCOUPLED, COUPLED,
            COUPLED_POSITION, and COUPLED_MOTION. The default value is UNCOUPLED.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        temperatureDependency
            A Boolean specifying whether the behavior data depend on temperature. The default value
            is OFF.
        frequencyDependency
            A Boolean specifying whether the behavior data depend on frequency. This value is
            applicable only if **behavior** =  LINEAR and **coupling** = UNCOUPLED. The default value is
            OFF.
        table
            A sequence of sequences of Floats specifying damping properties. Items in the table data
            are described below. The default value is an empty sequence.
        independentComponents
            A sequence of Ints specifying the list of independent components that are included in
            the definition of the connector damping data. This argument is applicable only if
            **behavior** = NONLINEAR and **coupling** = COUPLED_POSITION or COUPLED_MOTION. When this
            argument is applicable, at least one value must be specified. Only available components
            can be specified. The default value is an empty sequence.
        components
            A sequence of Ints specifying the components of relative motion for which the behavior
            is defined. Possible values are 1 ≤ **components** ≤ 6. Only available components can be
            specified. The default value is an empty sequence.

        Returns
        -------
        ConnectorDamping
            A ConnectorDamping object.

        Raises
        ------
        ValueError and TextError
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the ConnectorDamping object.

        Raises
        ------
        ValueError
        """
        ...



from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class OdbDataElementSet:
    """The OdbDataElementSet object stores element set data.

    .. note::
        This object can be accessed by::

            import visualization
            session.odbData[name].elementSets[i]
    """

    #: A String specifying the set name. This attribute is read-only.
    name: str = ""

    #: A String-to-tuple-of-Ints Dictionary specifying the elements in the set. This attribute
    #: is read-only.
    elements: str = ""



from __future__ import annotations

from .AFXDataComponent import AFXDataComponent
from .constants import DEFAULT_PAD
from .FXComposite import FXComposite
from .FXExponent import FXExponent
from .FXFont import FXFont
from .FXObject import FXObject
from .FXPacker import FXPacker

#: Value field is a string.
AFXTEXTFIELD_STRING: int = hash("AFXTEXTFIELD_STRING")

#: Value field is an integer.
AFXTEXTFIELD_INTEGER: int = hash("AFXTEXTFIELD_INTEGER")

#: Value field is a double.
AFXTEXTFIELD_FLOAT: int = hash("AFXTEXTFIELD_FLOAT")

#: Value fields consist of the real and imaginay components of a complex number.
AFXTEXTFIELD_COMPLEX: int = hash("AFXTEXTFIELD_COMPLEX")

#: Use a check button instead of a label.
AFXTEXTFIELD_CHECKBUTTON: int = hash("AFXTEXTFIELD_CHECKBUTTON")

#: Use a radio button instead of a label.
AFXTEXTFIELD_RADIOBUTTON: int = hash("AFXTEXTFIELD_RADIOBUTTON")

#: Orient label or button above text field.
AFXTEXTFIELD_VERTICAL: int = hash("AFXTEXTFIELD_VERTICAL")

#: Configure text field to the read-only state.
AFXTEXTFIELD_READONLY: int = hash("AFXTEXTFIELD_READONLY")

#: Allow IME (Japanese etc.) input.
AFXTEXTFIELD_IME: int = hash("AFXTEXTFIELD_IME")


class AFXTextField(FXPacker, AFXDataComponent):
    """This class contains a label that precedes a text field that allows the user to enter in text."""

    #: ID for setting imaginary values.
    ID_SETIMAGINARYVALUE: int = hash("ID_SETIMAGINARYVALUE")

    #: ID for getting imaginary values.
    ID_GETIMAGINARYVALUE: int = hash("ID_GETIMAGINARYVALUE")

    #: ID for the check/radio button.
    ID_BUTTON: int = hash("ID_BUTTON")

    #: ID for the text field.
    ID_TEXT: int = hash("ID_TEXT")

    #: ID for the text field with imaginary part.
    ID_IMG_TEXT: int = hash("ID_IMG_TEXT")

    def __init__(
        self,
        p: FXComposite,
        ncols: int,
        labelText: str,
        tgt: FXObject | None = None,
        sel: int = 0,
        opts: int = AFXTEXTFIELD_STRING,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = DEFAULT_PAD,
        pr: int = DEFAULT_PAD,
        pt: int = DEFAULT_PAD,
        pb: int = DEFAULT_PAD,
    ):
        """Constructor.

        Parameters
        ----------
        p : FXComposite
            Parent widget.
        ncols : int
            Number of columns.
        labelText : str
            Label string.
        tgt : FXObject | None
            Message target.
        sel : int
            Message ID.
        opts : int
            Options and hints.
        x : int
            X coordinate of origin.
        y : int
            Y coordinate of origin.
        w : int
            Width of the widget.
        h : int
            Height of the widget.
        pl : int
            Left padding (margin).
        pr : int
            Right padding (margin).
        pt : int
            Top padding (margin).
        pb : int
            Bottom padding (margin).
        """

    def create(self):
        """Creates the text field.

        Reimplemented from FXComposite.
        """

    def disable(self):
        """Disables the text field.

        Reimplemented from FXWindow.
        """

    def enable(self):
        """Enables the text field.

        Reimplemented from FXWindow.
        """

    def getCheck(self):
        """Returns the state of the check button or the radio button."""

    def getCursorPos(self):
        """Returns the cursor position."""

    def getDefaultWidth(self):
        """Returns the default width of the text field.

        Reimplemented from FXPacker.
        """

    def getExponentType(self):
        """Returns the exponent type of the text field for real and complex types."""

    def getImaginaryText(self):
        """Returns the imaginary text for the complex text field."""

    def getJustify(self):
        """Returns the text justification mode."""

    def getLabelFont(self):
        """Returns the label's font."""

    def getLabelText(self):
        """Returns the label text."""

    def getNumColumns(self):
        """Returns the number of columns."""

    def getPrecision(self):
        """Returns the precision of the text field for real and complex types."""

    def getText(self):
        """Returns the text."""

    def getValueType(self):
        """Returns the value type (AFXTEXTFIELD_FLOAT, etc.) of the text field."""

    def isEditable(self):
        """Returns True if the text in the input field may be edited."""

    def isReadOnlyState(self):
        """Returns True if the text field appears in the read-only state."""

    def isVerticalLayout(self):
        """Returns True if the layout orientation is vertical."""

    def setCheck(self, state: bool):
        """Sets the state of the check button or the radio button.

        Parameters
        ----------
        state : bool
            Check state.
        """

    def setCheckButtonSelector(self, sel: int):
        """Sets the message ID of the check button or the radio button.

        Parameters
        ----------
        sel : int
            Selector.
        """

    def setCheckButtonTarget(self, checkVal: bool = False):
        """Sets the message target of the check button or the radio button.

        Parameters
        ----------
        checkVal : bool
            Check state.
        """

    def setCursorPos(self, pos: int):
        """Sets the cursor position.

        Parameters
        ----------
        pos : int
            Position.
        """

    def setEditable(self, edit: bool = True):
        """Sets the editable state for the text field.

        Parameters
        ----------
        edit : bool
            If True, text can be edited.
        """

    def setExponentType(self, e: FXExponent):
        """Sets the exponent type of the text field for real and complex types.

        Parameters
        ----------
        e : FXExponent
            Exponent type.
        """

    def setFocus(self):
        """Moves the focus to the text field.

        Reimplemented from FXWindow.
        """

    def setFocusAndSelection(self):
        """Sets the focus to the input field and selects its contents."""

    def setFocusToCheckButton(self):
        """Moves the focus to the check button or the radio button (if existed) of the widget."""

    def setFocusToImaginaryTextField(self):
        """Moves the focus to the input field for the imaginary part."""

    def setFocusToTextField(self):
        """Moves the focus to the input field of the widget."""

    def setImaginaryFocusAndSelection(self):
        """Sets the focus to the input field for the imaginary part and selects its contents."""

    def setImaginaryText(self, text: str):
        """Sets the imaginary text for the complex text field.

        Parameters
        ----------
        text : str
            Imaginary text field text.
        """

    def setJustify(self, mode: int):
        """Sets the text justification mode.

        Parameters
        ----------
        mode : int
            Justification flag.
        """

    def setLabelFont(self, fnt: FXFont):
        """Sets the label's text font.

        Parameters
        ----------
        fnt : FXFont
            Label font.
        """

    def setLabelText(self, txt: str):
        """Sets the label text.

        Parameters
        ----------
        txt : str
            Label text.
        """

    def setNumColumns(self, cols: int):
        """Sets the number of columns. Note: The column width is based on the width of "m" of the font used.

        Parameters
        ----------
        cols : int
            Number of columns.
        """

    def setPrecision(self, p: int):
        """Sets the precision of the text field for real and complex types. Limitation: If an AFXTextField
        widget uses an AFXFloatKeyword object as its target, the widget must have AFXTEXTFIELD_FLOAT as one of
        its options for the precision setting to take effect.

        Parameters
        ----------
        p : int
            Precision.
        """

    def setReadOnlyState(self, readonly: bool = True):
        """Sets the read-only state of the text field.

        Parameters
        ----------
        readonly : bool
            Read-only state.
        """

    def setSelection(self, pos: int, len: int):
        """Select the specified number of characters starting at given position.

        Parameters
        ----------
        pos : int
            Position.
        len : int
            Length.
        """

    def setText(self, text: str):
        """Sets the text in the input field.

        Parameters
        ----------
        text : str
            Text field text.
        """

    def setValueType(self, type: int):
        """Sets the value type (AFXTEXTFIELD_FLOAT, etc.) of the text field.

        Parameters
        ----------
        type : int
            Value type.
        """

    def setVerticalLayout(self, vertical: bool):
        """Sets the layout orientation of the text field.

        Parameters
        ----------
        vertical : bool
            Vertical flag.
        """



from __future__ import annotations

from .AFXBoolKeyword import AFXBoolKeyword
from .AFXFileDialog import AFXFileDialog
from .AFXIntTarget import AFXIntTarget
from .AFXStringTarget import AFXStringTarget
from .FXWindow import FXWindow

#: A single file, existing or not (to save to).
AFXSELECTFILE_ANY: int = hash("AFXSELECTFILE_ANY")

#: An existing file (to load).
AFXSELECTFILE_EXISTING: int = hash("AFXSELECTFILE_EXISTING")

#: Multiple existing files.
AFXSELECTFILE_MULTIPLE: int = hash("AFXSELECTFILE_MULTIPLE")

#: Multiple existing files or directories.
AFXSELECTFILE_MULTIPLE_ALL: int = hash("AFXSELECTFILE_MULTIPLE_ALL")

#: Existing directory.
AFXSELECTFILE_DIRECTORY: int = hash("AFXSELECTFILE_DIRECTORY")


class AFXFileSelectorDialog(AFXFileDialog):
    """Abaqus."""

    def __init__(
        self,
        owner: FXWindow,
        title: str,
        pathNameTgt: AFXStringTarget,
        readOnlyKw: AFXBoolKeyword,
        mode: int = AFXSELECTFILE_ANY,
        patterns: str = "*",
        patternIndexTgt: AFXIntTarget | None = None,
    ):
        """Constructor typically used to create a dialog box that is posted from another dialog box (e.g. from a
        "Select..." button); a target is used for the pathName. If the dialog box allows multiple selection, the
        pathName target contains comma-separated path names of all selected files.

        Parameters
        ----------
        owner : FXWindow
            Owner
        title : str
            Dialog box title.
        pathNameTgt : AFXStringTarget
            Path name target.
        readOnlyKw : AFXBoolKeyword
            Read-only keyword.
        mode : int
            File selection mode.
        patterns : str
            File filter patterns.
        patternIndexTgt : AFXIntTarget | None
            Index used to select a file filter pattern when the dialog box is posted.
        """



from __future__ import annotations

from .constants import DECOR_ALL
from .FXApp import FXApp
from .FXIcon import FXIcon
from .FXTopWindow import FXTopWindow


class FXMainWindow(FXTopWindow):
    """Main application window."""

    def __init__(
        self,
        a: FXApp,
        name: str,
        ic: FXIcon | None = None,
        mi: FXIcon | None = None,
        opts: int = DECOR_ALL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = 0,
        pr: int = 0,
        pt: int = 0,
        pb: int = 0,
        hs: int = 0,
        vs: int = 0,
    ):
        """Construct a main window.

        Parameters
        ----------
        a : FXApp

        name : str

        ic : FXIcon | None

        mi : FXIcon | None

        opts : int

        x : int

        y : int

        w : int

        h : int

        pl : int

        pr : int

        pt : int

        pb : int

        hs : int

        vs : int
        """



from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import ENGINEERING, ON, Boolean, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class NumberFormat:
    """The NumberFormat object is a formatting template used to define formatting options for certain numeric
    output.This page discusses:Access[NumberFormat(...)](https://help.3ds.com/2022/english/DSSIMULIA_Established
    /SIMACAEKERRefMap/simaker-c-numberformatpyc.htm?ContextScope=all#simaker-
    numberformatnumberformatpyc)Members.

    .. note::
        This object can be accessed by::

            import visualization
            session.defaultFieldReportOptions.numberFormat
            session.fieldReportOptions.numberFormat
            session.journalOptions.defaultFormat
            session.journalOptions.fieldReportFormat
            session.journalOptions.geometryFormat
    """

    #: A Boolean specifying whether the printed digits should be padded with blank characters
    #: to ensure equal sized fields. The **blankPad** argument is useful when your printed output
    #: includes columns. The default value is ON.
    blankPad: Boolean = ON

    #: A SymbolicConstant specifying the formatting type. Possible values are ENGINEERING,
    #: SCIENTIFIC, and AUTOMATIC. The default value is ENGINEERING.
    format: SymbolicConstant = ENGINEERING

    #: An Int specifying the number of digits to be displayed in the result. **numDigits** >0>0.
    #: The default value is 6.
    numDigits: int = 6

    #: An Int specifying the number of decimal places to which the number is to be truncated
    #: for display. **precision** ≤0≤0. If **precision** =0, no truncation is applied. The default
    #: value is 0.
    precision: int = 0

    @abaqus_method_doc
    def __init__(
        self,
        blankPad: Boolean = ON,
        format: Literal[C.AUTOMATIC, C.ENGINEERING, C.SCIENTIFIC] = ENGINEERING,
        numDigits: int = 6,
        precision: int = 0,
    ):
        """This method creates a NumberFormat object.

        .. note::
            This function can be accessed by::

                session.defaultFieldReportOptions.NumberFormat
                session.fieldReportOptions.NumberFormat
                session.journalOptions.NumberFormat

        Parameters
        ----------
        blankPad
            A Boolean specifying whether the printed digits should be padded with blank characters
            to ensure equal sized fields. The **blankPad** argument is useful when your printed output
            includes columns. The default value is ON.
        format
            A SymbolicConstant specifying the formatting type. Possible values are ENGINEERING,
            SCIENTIFIC, and AUTOMATIC. The default value is ENGINEERING.
        numDigits
            An Int specifying the number of digits to be displayed in the result. **numDigits** >0>0.
            The default value is 6.
        precision
            An Int specifying the number of decimal places to which the number is to be truncated
            for display. **precision** ≤0≤0. If **precision** =0, no truncation is applied. The default
            value is 0.

        Returns
        -------
        NumberFormat
            A NumberFormat object.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .Profile import Profile


@abaqus_class_doc
class PipeProfile(Profile):
    """The PipeProfile object defines the properties of a circular pipe profile. The PipeProfile object is
    derived from the Profile object.

    .. note::
        This object can be accessed by::

            import section
            mdb.models[name].profiles[name]
            import odbSection
            session.odbs[name].profiles[name]

        The corresponding analysis keywords are:

        - BEAM SECTION
    """

    #: A String specifying the repository key.
    name: str

    #: A Float specifying the outer radius of the pipe. For more information, see [Beam
    #: cross-section
    #: library](https://help.3ds.com/2021/English/DSSIMULIA_Established/SIMACAEELMRefMap/simaelm-c-beamcrosssectlib.htm?ContextScope=all).
    r: float

    #: A Float specifying the wall thickness of the pipe.
    t: float

    @abaqus_method_doc
    def __init__(self, name: str, r: float, t: float):
        """This method creates a PipeProfile object.

        .. note::
            This function can be accessed by::

                mdb.models[name].PipeProfile
                session.odbs[name].PipeProfile

        Parameters
        ----------
        name
            A String specifying the repository key.
        r
            A Float specifying the outer radius of the pipe. For more information, see [Beam
            cross-section
            library](https://help.3ds.com/2021/English/DSSIMULIA_Established/SIMACAEELMRefMap/simaelm-c-beamcrosssectlib.htm?ContextScope=all).
        t
            A Float specifying the wall thickness of the pipe.

        Returns
        -------
        PipeProfile
            A PipeProfile object.

        Raises
        ------
        RangeError
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the PipeProfile object.

        Raises
        ------
        RangeError
        """
        ...
